<?php
 
class Favicon extends Model {
    protected static $_default_image = 'public/images/default_favicon.png';   // Relative to application root folder
    protected static $_local_image_storage = 'tmp/favicons';   // Relative to application root folder
    protected static $_convert_bin = '/home/futureof/ImageMagick-6.8.8-2/utilities/convert'; // full path to ImageMagick's convert utility    
    protected $_timeout = 86400;   // Timeout of favicon cache, in seconds

    protected $_virtualFields = array(
        'age' => 'NOW()-timestamp'
    );

    public static function defaultImage() {
        return self::$_default_image;
    }
    
    public static function getFaviconUrl($url) {
        return '/favicons/render/'.urlencode(str_replace("/", "|", $url));
                /*
        $p=parse_url($url);
        if ($p!==false) {
            //return $p['scheme'].'://'.$p['host'].'/favicon.ico';
            
        }
        return false;
                 * 
                 */
    }
    
    public function render_favico($url) {
        $ico = $this->_get_cached_ico($url);

        if (trim($ico)=="") $file = ROOT.DS.Favicon::$_default_image;
        else $file = ROOT.DS.Favicon::$_local_image_storage.DS.$ico;
        
        if (file_exists($file)) {
            $size = getimagesize($file);
            $fp = fopen($file, 'rb');

            if ($size and $fp) {
                header('Content-Type: '.$size['mime']);
                header('Content-Length: '.filesize($file));

                fpassthru($fp);
                exit;
            }
        }
        exit;
    }
    
    protected function _get_cached_ico($url) {
        $faviconUrl = '';
        
        $p = parse_url($url);
        if ($p!==false) {

            // Do not get icons for blacklisted sites
            $blackList = new Defaulticon();
            if ($blackList->blackListHost($p['host'])) return false;

            $find = new Favicon();
            $f = $find->find_by_field('site', $p['host']);
            if (count($f)) {
                if ($f[0]['Favicon']['age'] < $this->_timeout) return $f[0]['Favicon']['favicon'];
                else {
                    $del = new Favicon();
                    $del->id = $f[0]['Favicon']['id'];
                    $del->delete();
                }
            }
            
            $faviconUrl = $this->_download_and_store_ico($url);
        }
        
        return $faviconUrl;
    }
    
    protected function _download_and_store_ico($url) {
        $this->favicon = '';        
        
        $parsedUrl = parse_url($url);
        if ($parsedUrl) {
            $this->site = $parsedUrl['host'];
            $fiUrl = $this->_get_ico_url($url);

            $ch = curl_init($fiUrl);
            curl_setopt($ch,CURLOPT_HEADER,0);
            curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
            $image=curl_exec($ch);
            if (curl_getinfo($ch, CURLINFO_HTTP_CODE)=="200") {
                $pathName = ROOT.DS.'tmp'.DS.'favicons';
                if (file_exists($pathName) || mkdir($pathName)) {
                    $fileName = $this->site.'-favicon.ico';        
                    $handle = fopen($pathName.DS.$fileName, 'w');
                    fwrite($handle, $image);
                    fclose($handle);
                    
                    exec(Favicon::$_convert_bin.' "'.$pathName.DS.$fileName.'" -thumbnail 32x32 -quality 100 -alpha set -background none -flatten "'.$pathName.DS.$this->site.'-favicon.png"', $out, $result);
                    if ($result===0) {
                        $fileName = $this->site.'-favicon.png';
                    }

                    $this->favicon = $fileName;
                }
            }
            curl_close($ch);

            $favicon = $this->favicon;
            $this->save();
        }
        
        return $favicon;
    }
    
    protected function _get_ico_url($url) {
        $parsedUrl = parse_url($url);
        if ($parsedUrl) {        
            $fiUrl = $parsedUrl['scheme'] . '://' . $this->site . '/favicon.ico';

            $ch = curl_init($url);
            curl_setopt($ch,CURLOPT_HEADER,0);
            curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            $result=curl_exec($ch);
            if (curl_getinfo($ch, CURLINFO_HTTP_CODE)=="200") {
                $domDoc = DOMDocument::loadHTML($result);
                $xpath = new DOMXPath($domDoc);

                $query = '//link[@rel="icon" or @rel="shortcut icon"]/@href';
                foreach ($xpath->query($query) as $row) {                    
                    $urlP = parse_url($row->textContent);                    
                    if ($urlP) {
                        if (isset($urlP['scheme'])&&isset($urlP['host'])) {
                            // favicon href includes FQDN
                            $fiUrl = $row->textContent;
                        } else {
                            // favicon href is missing FQDN, using site's
                            $fiUrl = $parsedUrl['scheme'].'://'.$parsedUrl['host'];
                            if (substr($urlP['path'], 0, 1)=="/") {
                                // favicon href is absolute path
                                $fiUrl .= $urlP['path'];
                            } else {
                                // favicon href is relative path
                                $fiPathArr = explode("/", $parsedUrl['path']);
                                $end = array_pop($fiPathArr);
                                if (trim($end)=="") {
                                    $fiUrl .= $parsedUrl['path'];
                                } else {
                                    for($i=0;$i<count($parsedUrl['path']);$i++) {
                                        $fiUrl .= ($parsedUrl['path'][$i]==""?'/':$parsedUrl['path'][$i]);
                                    }
                                }
                                $fiUrl .= $row->textContent;
                            }
                        }
                    }
                }
            }
            curl_close($ch);    
            
            return $fiUrl;
        }
    }
}