/*
Dialog Window UI Component
/global/js/rm_platform/ui/rm_dialog.js
RLC 20101213
*/

define(['utils/rm_console', 'utils/rm_uniqueId', 
	'utils/rm_loadCss', 'utils/rm_indexOf',
	'utils/rm_checkHead'],
	function() {
		return rmDialog('/global/css/rm_css/rm_dialog.css');
	}
);

function rmDialog(style) {
	var m_console = new rmConsole('rmDialog');
	var dialog_src;
	var dialog_data;
	var dialog_opts;
	
	var dialog_width;
	var dialog_height;
	var dialog_left;
	var dialog_top;
	var dialog_id;
	var dialog_classes;
	var dialog_callback;
	var dialog_destroy;
	
	var container_id = 'rmDialogContainer';
	var loading_id = 'rmDialogLoading';
	var load_icon_id = 'rmDialogLoadIcon';
	var background_id = 'rmDialogBackground';
	
	var is_loading = false;
	var is_loaded = false;
	var is_visible = false;
	
	function init() {
		//load css
		if (typeof(style) !== 'undefined') {
			rmLoadCss().load(style);
		}
		//build dialog container, loader, and background
		if ($('#' + container_id).length > 0) {
			$('#' + container_id).remove();
		}
		$('body').append('<div id="' + container_id + '"></div>');
		if ($('#' + background_id).length == 0) {
			$('#' + container_id).after('<div id="' + background_id + '"></div>');
			$('#' + background_id).bind('click.rmDialog', hideDialog);
		}
		if ($('#' + loading_id).length == 0) {
			$('#' + container_id).after('<div id="' + loading_id + '"><img id="' + load_icon_id + '" src="/global/images/rm_images/rm_Icon_Loading_31-f0ab00_fff.gif" alt="loading..." /></div>');
		}
	}
	
	function refreshSettings() {
		//sourceType (string, url, jquery obj) - url
		//draggable - false
		//dragTarget - undefined
		//dropShadowClass - rmDialogShadow
		//position {top, left} - centered
		//dimensions {width, height}
		//backgroundColor - #999 (from css)
		//backgroundOpacity - 0.7 (thanks jquery)
		//allowMultiple - false
		
		//set dimensions
		if (typeof(dialog_opts.dimensions) !== 'undefined' && typeof(dialog_opts.dimensions.width) !== 'undefined' && typeof(dialog_opts.dimensions.height) !== 'undefined') {
			dialog_width = dialog_opts.dimensions.width;
			dialog_height = dialog_opts.dimensions.height;
		} else {
			//use this to test setting and revert to dims based on html + css loaded at runtime
			dialog_width = -1;
			dialog_height = -1;
		}
		
		//TODO: implement custom modal ids and classes
		if (typeof(dialog_opts.id) !== 'undefined') {
			dialog_id = dialog_opts.id;
		} else {
			dialog_id = '';
		}
		
		//set callback to fire when modal is visible
		if (typeof(dialog_opts.callback) !== 'undefined') {
			dialog_callback = dialog_opts.callback;
		}
		
		//set destroy. will remove contents of modal on close.
		if (typeof(dialog_opts.destroy) !== 'undefined') {
			dialog_destroy = dialog_opts.destroy;
		} else {
			dialog_detroy = false;
		}
	}
	
	function startLoad() {
		var load_left;
		var load_top;
		$('#' + loading_id).show();
		if ($(window).scrollTop() > 0 || $(window).scrollLeft() > 0) {
            load_left = $(window).width() / 2 - $('#' + loading_id).width() / 2 + $(window).scrollLeft();
			load_top = $(window).height() / 2 - $('#' + loading_id).height() / 2 + $(window).scrollTop();
        } else {
            load_left = $(window).width() / 2 - $('#' + loading_id).width() / 2;
            load_top = $(window).height() / 2 - $('#' + loading_id).height() / 2;
        }
		$('#' + loading_id).css({
			'top': load_top + 'px',
			'left': load_left + 'px'
		});
		$('#' + background_id).show();
	}
	
	function loadContent() {
		//if sourceType == url
		$.ajax({
			url: dialog_src,
			cache: false,
			dataType: 'html',
			success: contentReceived,
			error: contentError
		});
		//else sourceType == string or jquery obj
	}
	
	function buildDialog() {
		var temp_obj = $('<div>' + dialog_data + '</div>');
		if (temp_obj.find('link').length > 0) {
			rmCheckHead().add(temp_obj.find('link'));
		}
		if (temp_obj.find('script').length > 0) {
			rmCheckHead().add(temp_obj.find('script'));
		}
		$('#' + container_id).html(temp_obj.html());
	}
	
	function showDialog() {
		$('#' + background_id).show();
		$('#' + container_id).show();
		positionDialog();
		$(window).bind('scroll', positionDialog);
		$(window).bind('resize', positionDialog);
		is_visible = true;
	}
	
	function hideDialog() {
		$('#' + container_id).hide();
		$('#' + background_id).hide();
		$(window).unbind('scroll', positionDialog);
		$(window).unbind('resize', positionDialog);
		//this never seemed to work, see where this is bound in this.load()
		$(window).unbind('received.rmDialog.load', showDialog);
		is_visible = false;
		if (dialog_destroy) {
			$('#' + container_id).empty();
		}
	}
	
	//events
	function contentReceived(data, status, xhr) {
		is_loading = false;
		is_loaded = true;
		dialog_data = data;
		buildDialog();
		$('#' + loading_id).hide();
		$(window).trigger('received.rmDialog.load');
		if (typeof(dialog_callback) !== 'undefined') {
			(dialog_callback)();
		}
	}
	
	function contentError(evt) {
		
	}
	
	function positionDialog() {
		if ($(window).scrollTop() > 0 || $(window).scrollLeft() > 0) {
            positionToScroll();
        } else {
            positionToWindow();
        }
	}
	
	function positionToScroll() {
		m_console.log('pos to scroll');
		var scroll_target;
		var scroll_left = $(window).scrollLeft();
		var scroll_top = $(window).scrollTop();
		
		if (is_loading) {
			scroll_target = $('#' + loading_id);
			dialog_left = $(window).width() / 2 - scroll_target.width() / 2 + scroll_left;
			dialog_top = $(window).height() / 2 - scroll_target.height() / 2 + scroll_top;
		} else {
			scroll_target = $('#' + container_id);
			if (dialog_width == -1 && dialog_height == -1) {
				dialog_width = scroll_target.width();
				dialog_height = scroll_target.height();
			}
			dialog_left = $(window).width() / 2 - dialog_width / 2 + scroll_left;
			dialog_top = $(window).height() / 2 - dialog_height / 2 + scroll_top;
		}
		scroll_target.css({
			'left': dialog_left + 'px',
			'top': dialog_top + 'px'
		});
		$('#' + background_id).css({
			'left': scroll_left + 'px',
			'top': scroll_top + 'px'
		});
	}
	
	function positionToWindow() {
		var window_target;
		if (is_loading) {
			window_target = $('#' + loading_id);
		} else {
			window_target = $('#' + container_id);
		}
		dialog_width = window_target.width();
		m_console.log('width: ' + dialog_width);
		dialog_height = window_target.height();
		m_console.log('height: ' + dialog_height);
		dialog_left = $(window).width() / 2 - dialog_width / 2;
		dialog_top = $(window).height() / 2 - dialog_height / 2;
		window_target.css({
			'left': dialog_left,
			'top': dialog_top
		});
		$('#' + background_id).css({
			'left': '0px',
			'top': '0px'
		});
	}
	
	//public
	this.load = function(src, opts) {
		var is_bound = false;
		is_loading = true;
		is_loaded = false;
		dialog_opts = (typeof(opts) !== 'undefined') ? opts : {};
		dialog_src = src;
		refreshSettings();
		startLoad();
		loadContent();
		//unbinding this wasn't working. also, notice the namespace gets reversed. maybe a jquery bug?
		if (typeof($(window).data('__events__')) !== 'undefined') {
			for (var i in $(window).data('__events__').events.received) {
				if ($(window).data('__events__').events.received[i].namespace == 'load.rmDialog') {
					is_bound = true;
				}
			}
		}
		if (!is_bound) {
			$(window).bind('received.rmDialog.load', showDialog);
		}
	}
	
	this.toggle = function() {
		if (is_visible) {
			hideDialog();
		} else {
			refreshSettings();
			showDialog();
		}
	}
	
	this.hide = function() {
		hideDialog();
	}
	
	this.show = function() {
		showDialog();
	}
	
	this.isLoaded = function() {
		return is_loaded;
	}
	
	init();
	return this;
}