<?php

class FormValidator {
    protected $_methods = array(
        "text" => "validateTextField",
        "email"=>"validateEmailField",
        "date"=>"validateDateField",
        "numeric"=>"validateNumericField"
    );
    protected $_fields = array();
    protected $_errors = array();
    protected $_required = array();
    protected $_labels = array();
    protected $_currentId = false;

    public function getFailedFields() {
        return array_keys($this->_errors);
    }
    public function getErrors() {
        $err = 'The following errors were found:<br><ul>';
        foreach(array_keys($this->_errors) as $k) {
            $err .= '<li>'.$this->_labels[$k].': '.$this->_errors[$k].'</li>';
        }
        $err .= '</ul>';        
        return $err;
    }
    
    public function addMethod($type, $method) {
        $this->_methods[$type] = $method;
    }
    
    public function addFields($obj=array()) {        
        foreach (array_keys($obj) as $field) {
            $opts = array(
                'required'=>(isset($obj[$field]['required'])?$obj[$field]['required']:false), 
                'label'=>(isset($obj[$field]['label'])?$obj[$field]['label']:'')
            );
            $this->add($field, $obj[$field]['type'], $opts);
        }
    }
    
    public function add($field, $type, $obj=array()) {
        $this->_fields[$field] = $type;
        $this->_required[$field] = (isset($obj['required'])?$obj['required']:false);
        $this->_labels[$field] = (isset($obj['label'])?$obj['label']:'');
    }
    
    public function validate($data=array(), $currentId=false) {
        $this->_currentId = $currentId;
        $go = true;
        foreach (array_keys($this->_fields) as $f) {
            $ret = $this->validateField($this->_fields[$f], $data[$f], $this->_required[$f]);
            if ($ret['status']!=true) {
                $go = false;
                $this->_errors[$f] = $ret['errors'];
            }
        }
        return $go;
    }
    
    public function validateField($type, $data, $required) {
        $method = $this->_methods[$type];
        $resp = $this->$method($data, $required);
        return array("status"=>($resp['status']==true?true:false), "errors"=>$resp['errors']);
    }
    
    public function validateTextField($data, $required) {
        $out = array("status"=>true, 'errors'=>'');
        if ( $required && ( (!is_array($data) && trim($data) == "") || (is_array($data) && !count($data)) )){
            $out = array("status"=>false, "errors"=>"Required field");
        }
        return $out;        
    }
    
    public function validateEmailField($data, $required) {
        $out = array("status"=>true, 'errors'=>'');
        $data = trim($data);
        if ($required==true && $data=="") {
            $out = array("status"=>false, "errors"=>"Required field");
        } else if (!preg_match("/^[a-zA-Z0-9_\.\-]+@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/", $data)) {
            $out = array("status"=>false, "errors"=>"Invalid email address");
        }

        return $out;
    }
    
    public function validateDateField($data, $required) {
        $out = array("status"=>true, 'errors'=>'');
        $data = trim($data);
        if ($required==true && $data=="") {
            $out = array("status"=>false, "errors"=>"Required field");
        } else {
            if (preg_match('|(\d+)/(\d+)/(\d+)|', $data, $matches)) {
                if (!checkdate($matches[1], $matches[2], $matches[3])) {
                    $out = array("status"=>false, "errors"=>"Invalid date");
                }                
            } else {
                $out = array("status"=>false, "errors"=>"Invalid date format");
            }
        }

        return $out;        
    }
    
    public function validateNumericField($data, $required) {
        $out = array("status"=>true, 'errors'=>'');
        $data = trim($data);
        if ($required==true && $data=="") {
            $out = array("status"=>false, "errors"=>"Required field");
        } else if (!is_numeric($data)) {
            $out = array("status"=>false, "errors"=>"Not a numeric field");
        }

        return $out;        
    }
}
